@@ -1,8 +1,8 @@
+import { defineMessages } from 'react-intl';
 import { normalize, arrayOf } from 'normalizr';
-import { fromJS, List } from 'immutable';
 import when from 'when';
-import { reduce } from 'lodash';
 
+import { getNodesByIds } from '../selectors/nodes';
 import IronicApiErrorHandler from '../services/IronicApiErrorHandler';
 import IronicApiService from '../services/IronicApiService';
 import MistralApiService from '../services/MistralApiService';
@@ -12,6 +12,22 @@ import NotificationActions from './NotificationActions';
 import { nodeSchema } from '../normalizrSchemas/nodes';
 import MistralConstants from '../constants/MistralConstants';
 import logger from '../services/logger';
+import { setNodeCapability } from '../utils/nodes';
+
+const messages = defineMessages({
+  introspectionNotificationMessage: {
+    id: 'NodesActions.introspectionNotificationMessage',
+    defaultMessage: 'Selected nodes were successfully introspected.'
+  },
+  introspectionNotificationTitle: {
+    id: 'NodesActions.introspectionNotificationTitle',
+    defaultMessage: 'Nodes Introspection Complete'
+  },
+  introspectionFailedNotificationTitle: {
+    id: 'NodesActions.introspectionFailedNotificationTitle',
+    defaultMessage: 'Nodes Introspection Failed'
+  }
+});
 
 export default {
   startOperation(nodeIds) {
@@ -41,35 +57,24 @@ export default {
     };
   },
 
-  receiveNodes(nodes) {
+  receiveNodes(entities) {
     return {
       type: NodesConstants.RECEIVE_NODES,
-      payload: nodes
+      payload: entities
     };
   },
 
   fetchNodes() {
     return (dispatch, getState) => {
       dispatch(this.requestNodes());
-      IronicApiService.getNodes().then((response) => {
-        return when.all(response.nodes.map((node) => {
-          return IronicApiService.getNode(node.uuid);
-        }));
-      }).then((nodes) => {
-        const normalizedNodes = normalize(nodes, arrayOf(nodeSchema)).entities.nodes || {};
-        return when.map(Object.keys(normalizedNodes), nodeUUID => {
-          return IronicApiService.getNodePorts(nodeUUID).then(response => {
-            const macs = reduce(response.ports, (result, value) => {
-              result.push(value.address);
-              return result;
-            }, []).join(', ');
-            return {
-              nodeUUID,
-              macs
-            };
+      IronicApiService.getNodes().then(response => {
+        return when.map(response.nodes, node => {
+          return IronicApiService.getNodePorts(node.uuid).then(response => {
+            node.portsDetail = response.ports;
+            return node;
           });
-        }).then(values => {
-          values.forEach(v => normalizedNodes[v.nodeUUID].macs = v.macs);
+        }).then(nodes => {
+          const normalizedNodes = normalize(nodes, arrayOf(nodeSchema)).entities;
           dispatch(this.receiveNodes(normalizedNodes));
         });
       }).catch((error) => {
@@ -124,9 +129,9 @@ export default {
   },
 
   nodesIntrospectionFinished(messagePayload) {
-    return (dispatch, getState) => {
-      const payload = fromJS(messagePayload);
-      const nodeIds = payload.get('introspected_nodes').keys();
+    return (dispatch, getState, { getIntl }) => {
+      const { formatMessage } = getIntl(getState());
+      const nodeIds = messagePayload.execution.input.node_uuids;
       dispatch(this.finishOperation(nodeIds));
       dispatch(this.fetchNodes());
 
@@ -134,19 +139,16 @@ export default {
       case 'SUCCESS': {
         dispatch(NotificationActions.notify({
           type: 'success',
-          title: 'Nodes Introspection Complete',
-          message: 'Selected nodes were successfully introspected'
+          title: formatMessage(messages.introspectionNotificationTitle),
+          message: formatMessage(messages.introspectionNotificationMessage)
         }));
         break;
       }
       case 'FAILED': {
-        const nodeErrors = payload.get('introspected_nodes').reduce((nodeErrors, value, key) => {
-          return nodeErrors.push(`${key}: ${value.get('error')}`);
-        }, List());
         dispatch(NotificationActions.notify({
           type: 'error',
-          title: payload.get('message'),
-          message: nodeErrors.toArray().join(', ')
+          title: formatMessage(messages.introspectionFailedNotificationTitle),
+          message: messagePayload.message.join(', ')
         }));
         break;
       }
@@ -156,6 +158,22 @@ export default {
     };
   },
 
+  tagNodes(nodeIds, tag) {
+    return (dispatch, getState) => {
+      const nodes = getNodesByIds(getState(), nodeIds);
+      nodes.map(node => {
+        dispatch(this.updateNode({
+          uuid: node.get('uuid'),
+          patches: [{
+            op: 'replace',
+            path: '/properties/capabilities',
+            value: setNodeCapability(node.getIn(['properties', 'capabilities']), 'profile', tag)
+          }]
+        }));
+      });
+    };
+  },
+
   startProvideNodes(nodeIds) {
     return (dispatch, getState) => {
       dispatch(this.startOperation(nodeIds));
@@ -194,11 +212,13 @@ export default {
         break;
       }
       case 'FAILED': {
-        dispatch(NotificationActions.notify({
-          type: 'error',
-          title: 'Error',
-          message: messagePayload.message
-        }));
+        messagePayload.message.map(message => {
+          dispatch(NotificationActions.notify({
+            type: 'error',
+            title: 'Error',
+            message: message.result
+          }));
+        });
         break;
       }
       default:
@@ -247,7 +267,6 @@ export default {
   deleteNodes(nodeIds) {
     return (dispatch, getState) => {
       dispatch(this.startOperation(nodeIds));
-      dispatch(this.pollNodeslistDuringProgress());
       nodeIds.map((nodeId) => {
         IronicApiService.deleteNode(nodeId).then(response => {
           dispatch(this.deleteNodeSuccess(nodeId));
